#ifndef __CAR_H__
#define __CAR_H__

#include "track.h"

#define MODE_STATIC	0	/* static friction */
#define MODE_SLIP	1	/* kinetic friction */
#define MODE_AIR	2	/* airborne */

#define TIRE_RADIUS	0.45
#define TIRE_CIRCUMF	(TIRE_RADIUS * 2 * PI)

#define SHOCK_KP	1200
#define SHOCK_KD	60
#define SHOCK_L0	TIRE_RADIUS

struct wheel {
  VEC pos;		// 3D position in WCS
  int mode;		// is it in slip mode? airborne?
  double dir;		// direction wheel is pointing in  (in degrees)
  double spin;		// spin rate
  double markerPos;	// marker on wheel position
  double Fwheel;	// the driving force the wheel is applying to ground
  int last_seg;		// index of last track_segment that the wheel was in

  // shocks
  double shock_kp;
  double shock_kd;
  double shock_l0;	// spring rest length
  double shock_prev_l;	// spring length from last iteration

  // closest ground info cache
  VEC surf_normal;	// normalized!!
  double dist;		// distance to ground
  VEC closest_pnt;	// closest point on the ground to tire patch
  VEC v_net_u;		// normalized net velocity vector of tire patch 
  			// w.r.t. ground, and perpendicular to surface normal
  VEC shock_force;

  
  wheel():
      mode(MODE_AIR), dir(0), spin(0), markerPos(0), Fwheel(0),
      shock_kp(SHOCK_KP), shock_kd(SHOCK_KD), shock_l0(SHOCK_L0), 
      shock_prev_l(SHOCK_L0), last_seg(0) {
  }

  // normalized, relative to ground
  bool get_patch_vel( const VEC& car_vel, VEC car_forward, const VEC& car_up );

  // 
  bool get_wheel_dir( VEC car_fwd, const VEC& car_up, VEC& result );

  // returns the driving force vector (perpendicular to surf_normal)
  bool get_F_wheel_vec( const VEC& car_fwd, const VEC& car_up, VEC& result );

  // calculates shock force
  void calc_F_shock();

  // calculates the closest point info
  bool get_closest_info();

  // calculates the new spin rate of the wheel
  void calc_spin( const VEC& car_vel, const VEC& car_fwd );
};

void draw_car();

//////////////////////////////// wheel order
// wheels[0] = front left
// wheels[1] = front right
// wheels[2] = back left
// wheels[3] = back right
extern wheel wheels[4];


//////////////////////////////// car specs
// modelled as a rectangular solid:
// 	front to back: 4m
// 	side to side: 2m
// 	top to bottom: 1m
#define CAR_FB		4
#define CAR_SS		2
#define CAR_TB		1


#endif // __CAR_H__
