#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "glaux.h"

#include "track.h"

track_seg seg( 20, 0, 0, 0, FVEC(0,0,0), FVEC(0,1,0) );



/*********************************************************
    PROC: myinit()
    DOES: performs most of the OpenGL intialization -- don't change
**********************************************************/

void myinit(void)
{
  GLfloat ambient[] = { 0.0, 0.0, 0.0, 1.0 };
  GLfloat diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat specular[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat position[] = { 0.0, 3.0, 3.0, 0.0 };

  GLfloat lmodel_ambient[] = { 0.2, 0.2, 0.2, 1.0 };
  GLfloat local_view[] = { 0.0 };

  GLfloat mat[4];

  /**** set lighting parameters ****/
  glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
  glLightfv(GL_LIGHT0, GL_POSITION, position);
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
  glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, local_view);

  /*    glFrontFace (GL_CW); */
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_AUTO_NORMAL);
  glEnable(GL_NORMALIZE);
  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LESS);
}

/*********************************************************
    PROC: set_colour();
    DOES: draws a teapot of the given colour -- don't change
**********************************************************/

void set_colour(float r, float g, float b)
{
  float ambient = 0.2;
  float diffuse = 0.7;
  float specular = 0.4;
  GLfloat mat[4];
  /**** set ambient lighting parameters ****/
  mat[0] = ambient*r;
  mat[1] = ambient*g;
  mat[2] = ambient*b;
  mat[3] = 1.0;
  glMaterialfv (GL_FRONT, GL_AMBIENT, mat);

  /**** set diffuse lighting parameters ******/
  mat[0] = diffuse*r;
  mat[1] = diffuse*g;
  mat[2] = diffuse*b;
  mat[3] = 1.0;
  glMaterialfv (GL_FRONT, GL_DIFFUSE, mat);

  /**** set specular lighting parameters *****/
  mat[0] = specular*r;
  mat[1] = specular*g;
  mat[2] = specular*b;
  mat[3] = 1.0;
  glMaterialfv (GL_FRONT, GL_SPECULAR, mat);
  glMaterialf (GL_FRONT, GL_SHININESS, 0.5);
}

/*********************************************************
    PROC: display()
    DOES: this gets called by the event handler to draw
          the scene, so this is where you need to build
	  your scene -- make your changes and additions here
**********************************************************/

void display(void)
{
  /* glClearColor (red, green, blue, alpha           */
  /* Ignore the meaning of the 'alpha' value for now */
  glClearColor(0,0,0.2,1.0);   /* set the background colour to black */
  /* OK, now clear the screen with the background colour */
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  // fire it off in WCS
  set_colour( 0.3, 0.3, 0.3 );
  seg.draw();

  // done
  glFlush();
}

/*********************************************************
    PROC: myReshape()
    DOES: handles the window being resized -- don't change
**********************************************************/

void myReshape(int w, int h) {
  glViewport(0, 0, w, h);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  /*** this defines the field of view of the camera   ***/
  /*** Making the first 4 parameters larger will give ***/
  /*** a larger field of view, therefore making the   ***/
  /*** objects in the scene appear smaller            ***/
  glFrustum(-1,1,-1,1,1,100);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  /*** this sets the virtual camera          ***/
  /*** gluLookAt( x,y,z,   x,y,z   x,y,z );  ***/
  /***            camera  look-at camera-up  ***/
  /***            pos'n    point   vector    ***/

  gluLookAt(-5,-10,10,0,0,0,0,1,0);
}


 /*********************************************************
     PROC: main()
     DOES: calls initialization, then hands over control
	   to the event handler, which calls 
	   display() whenever the screen needs to be redrawn
 **********************************************************/

int main(int argc, char** argv) {
  auxInitDisplayMode (AUX_SINGLE | AUX_RGB | AUX_DEPTH);
  auxInitPosition (0, 0, 300, 300);
  auxInitWindow (argv[0]);
  myinit();
  auxReshapeFunc (myReshape);
  printf("To quit:  Hit 'ESC' when the mouse is over the window\n");
  auxMainLoop(display);
}

