#ifndef __TRACK_H__
#define __TRACK_H__

#include <stdlib.h>

#include "vec_math.h"

class track_seg {
public:
  track_seg( 	double len, double rad, 
		double t1, double t2,
		double ornt, VEC p, VEC tngnt):
      length(len), radius(rad), orient(ornt), tilt_init(t1), tilt_fin(t2), 
      pos(p), dir_init(tngnt)
    {
      dir_init.normalize();
    }

  double get_tilt( double len );
  VEC get_orient_norm();
  bool get_normal_tangent( double len, VEC& n, VEC& t );
  bool get_pos( double len, VEC& p );
  bool get_pos_extent( double len, VEC& p, VEC& right );
  double get_sweep();
  void draw();

  int get_closest_info( const VEC& p1, VEC& p2, double& dist, VEC& normal );

// I know what I am doing so no point protecting this stuff;
// it will only make the code more complicated later on
public:
  double length;	// (arc)length of the segment
  double radius;	// curvature radius (0==straight)
  double orient;	// orientation (degrees) of the curve:
			// 0=segment curves vertically up, if looking forward
			// 180=seg curves vert. down, i.l.f.
			// 90=seg curves horiz right, i.l.f.
			// -90=270=seg curves horiz left, i.l.f.
			//    forward = "in the direction of tangent"
			//    with up being closest vector to (0,0,1)
  double tilt_init;	// tilt at beginning of segment (degrees)
  double tilt_fin;	// tilt at end of segment (degrees)
  VEC pos;		// starting 3D position
  VEC dir_init;	// initial tangent

protected:
  void get_untilted_normal_tangent( double len, VEC& n, VEC& t );
};


class track {
public:
  track(): segs(NULL), n(0)
  {}
  track( const char* szFile ) {
    load( szFile );
  }

  int num_of_segs()	{ return n; }

  bool load( const char* szFile );
  void draw() {
    if( segs )
      for( int i=0; i<n; i++ )
	segs[i]->draw();
  }

  track_seg* get_seg( int i ) {
    if( i<n && i>=0 )
      return segs[i];
    else
      return NULL;
  }

protected:
  int	n;
  track_seg** segs;
};


#ifndef PI
  #ifdef M_PI
    #define PI M_PI
  #else
    #define PI 3.141593
  #endif
#endif

#define DEGS_TO_RADS(x)	(((double)x)/180*PI)
#define RADS_TO_DEGS(x)	(((double)x)/PI*180)


#endif // __TRACK_H__
