/************************************************************
	FILE:		user.C
	CONTAINS:	user functions
************************************************************/

#include "glaux.h"

#include "lib_defs.h"
#include "lib_globals.h"
#include "lib_draw.h"
#include "lib_user.h"

#include "user.h"      // add your prototypes here
#include "sim.h"
#include "car.h"

// undefine below if you want to save each frame to a ppm
//#define RECORD_ON



track trk( "test.track" );

float Sim_time=0.0;
float Disp_time=0.0;
double Dtsim=0.01;
double Dtdisp=0.01;
float Y=0;
int Sim = 0;
float State[100];      // preallocate a large state vector


/*********************************************************
    PROC: set_colour();
    DOES: draws a teapot of the given colour -- don't change
**********************************************************/

void set_colour(float r, float g, float b)
{
  float ambient = 0.1;
  float diffuse = 0.7;
  float specular = 0.4;
  GLfloat mat[4];
  /**** set ambient lighting parameters ****/
  mat[0] = ambient*r;
  mat[1] = ambient*g;
  mat[2] = ambient*b;
  mat[3] = 1.0;
  glMaterialfv (GL_FRONT, GL_AMBIENT, mat);

  /**** set diffuse lighting parameters ******/
  mat[0] = diffuse*r;
  mat[1] = diffuse*g;
  mat[2] = diffuse*b;
  mat[3] = 1.0;
  glMaterialfv (GL_FRONT, GL_DIFFUSE, mat);

  /**** set specular lighting parameters *****/
  mat[0] = specular*r;
  mat[1] = specular*g;
  mat[2] = specular*b;
  mat[3] = 1.0;
  glMaterialfv (GL_FRONT, GL_SPECULAR, mat);
  glMaterialf (GL_FRONT, GL_SHININESS, 0.5);
}

//////////////////////////////////////////////////
//	PROC:	user_gl_init(void)
//	DOES:	Initializes and OpenGL event handling
//          Probably not too useful, but you never know...
//////////////////////////////////////////////////

void user_gl_init(void)
{
  gluLookAt(4,6,20,0,0,0,0,1,0);

  void key_toggle_view(void);       // defined later in this file

      // when V-key is pressed in OpenGL window,
      // call key_toggle_view()
  auxKeyFunc (AUX_v, key_toggle_view);



  GLfloat ambient[] = { 0.2, 0.2, 0.2, 1.0 };
  GLfloat diffuse[] = { 1, 1, 1, 1.0 };
  GLfloat specular[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat position[] = { 1, 1, 1, 0.0 };

  //    GLfloat lmodel_ambient[] = { 0.1, 0.1, 0.1, 1.0 };
  //    GLfloat local_view[] = { 0.0 };

  GLfloat mat[4];

  glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR, specular);
  glLightfv(GL_LIGHT0, GL_POSITION, position);
  //    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
  //    glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, local_view);

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_AUTO_NORMAL);
  glEnable(GL_NORMALIZE);
  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LESS);
}

//////////////////////////////////////////////////
//	PROC:	user_init(void)
//	DOES:	Initializes user-defined globals
//////////////////////////////////////////////////

void user_init(void)
{
	Tcl_CreateCommand(Interp, "sim_start", sim_start, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(Interp, "sim_stop", sim_stop, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(Interp, "sim_reset", sim_reset, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_LinkVar(Interp, "dtsim", (char *) &Dtsim, TCL_LINK_DOUBLE);
	Tcl_LinkVar(Interp, "dtdisp", (char *) &Dtdisp, TCL_LINK_DOUBLE);
}

///////////////////////////////////////////////////
//	PROC:	user_loop(void)
//	DOES:	All the action happens here. 
//            Queries for both OpenGL and TK events
///////////////////////////////////////////////////

void user_loop(void)
{
  extern int Quit;
  extern void check_events(void);
  extern Tcl_Interp *Interp;
  STR string;
  STR command;
  int count=0;

  Tcl_EvalFile(Interp, "ui");
  dyn_init_state();      // initialize state
  while (1) {
	if (Sim) {
//	  printf("Dtsim=%6.3d\n",Dtsim);
	  Sim_time += Dtsim;
	  sprintf(string, "%f", Sim_time);
	  Tcl_SetVar(Interp, "time", string, 0);
	  dyn_sim();                  // simulate one step
	  if (Sim_time>Disp_time) {
		display();
		Disp_time += Dtdisp; 
#ifdef RECORD_ON
		sprintf(command,"img_save /var/tmp/images/img%d.ppm",count);
		count++;
		Tcl_Eval(Interp,command);
#endif
	  }
	}	
	check_events();
  }
}

////////////////////////////////////////////////////
//	PROC:	user_draw(void)
//	DOES:	user's drawing code
////////////////////////////////////////////////////

#define GRID_SP	1
void user_draw(void)
{
  FVEC position;
  position[0] = -10;
  position[1] = -10;
  position[2] = 200;
  glLightfv(GL_LIGHT0, GL_POSITION, position);

  // draw a grid
  set_colour(1,1,1);
  FVEC mat; mat[0] = mat[1] = mat[2] = mat[3] = 1.0;
  glMaterialfv (GL_FRONT, GL_AMBIENT, mat);
  for( int x=-10*GRID_SP; x<=10*GRID_SP; x+=GRID_SP ) {
    glBegin(GL_LINES);
      glVertex3f( x, -10*GRID_SP, 0 );
      glVertex3f( x, 10*GRID_SP, 0 );
      glVertex3f( -10*GRID_SP, x, 0 );
      glVertex3f( 10*GRID_SP, x, 0 );
    glEnd();
  }

  // draw an UP vector at the origin
  set_colour(1.0,0,0);
  auxSolidSphere(0.2);
  glPushMatrix();
  glTranslatef(0,0,0.15);
  auxSolidSphere(0.1);
  glPopMatrix();

  // draw the track
  set_colour( 0.5, 0.5, 0.5 );
  trk.draw();

  // draw the car
  draw_car();

  // draw the closest_pnt's
  set_colour(1.0,1.0,0.0);		// yellow
  for( int w=0; w<4; w++ ) {
    glPushMatrix();
    glTranslatef( wheels[w].closest_pnt[0], wheels[w].closest_pnt[1], 
		wheels[w].closest_pnt[2] );
    auxSolidSphere(0.1);
    glPopMatrix();
  }

}


/*********************************************************
    PROC: key_toggle_view()
    DOES: toggles between rotate & translate modes
**********************************************************/

void key_toggle_view()
{
  switch(Mode) {
    case TRANSLATE_MODE:
      Mode = ROTATE_MODE;
      break;
    case ROTATE_MODE:
      Mode = TRANSLATE_MODE;
      break;
  }
  display();
}


///////////////////////////////////////////////////////////
//    PROC: sim_start()
//    DOES: starts dynamic simulation
///////////////////////////////////////////////////////////

int sim_start(ClientData clientData,
		   Tcl_Interp *interp,
		   int argc, char **argv)
{
  printf("Simulation started\n");
  Sim = 1;
  return TCL_OK;
}

///////////////////////////////////////////////////////////
//    PROC: sim_stop()
//    DOES: stops dynamic simulation
///////////////////////////////////////////////////////////

int sim_stop(ClientData clientData,
		   Tcl_Interp *interp,
		   int argc, char **argv)
{
  printf("Simulation stopped\n");
  Sim = 0;
  return TCL_OK;
}

///////////////////////////////////////////////////////////
//    PROC: sim_reset()
//    DOES: resets state of dynamic simulation
///////////////////////////////////////////////////////////

int sim_reset(ClientData clientData,
		   Tcl_Interp *interp,
		   int argc, char **argv)
{
  printf("reset state\n");
  dyn_init_state();             // initialize state
  display();
  return TCL_OK;
}

