
#include <iostream.h>
#include <math.h>

#include "vec_math.h"

inline
t_elem
VEC::size() const {
  return sqrt( data[0]*data[0] + data[1]*data[1] + data[2]*data[2] );
}

void
VEC::add( const VEC& a, const VEC& b ) {
  for( int i=0; i<3; i++ )
    data[i] = a.data[i] + b.data[i];
}

void
VEC::sub( const VEC& a, const VEC& b ) {
  for( int i=0; i<3; i++ )
    data[i] = a.data[i] - b.data[i];
}

void
VEC::smult( t_elem d ) {
  for( int i=0; i<3; i++ )
    data[i] *= d;
}

void
VEC::normalize() {
  t_elem d=size();
  for( int i=0; i<3; i++ )
    data[i] /= d;
}

void
VEC::xprod( const VEC& a, const VEC& b ) {
  data[0] = a.data[1]*b.data[2] - a.data[2]*b.data[1];
  data[1] = a.data[2]*b.data[0] - a.data[0]*b.data[2];
  data[2] = a.data[0]*b.data[1] - a.data[1]*b.data[0];
}

void
VEC::print() {
  cout << data[0] << "," << data[1] << "," << data[2] << endl;;
}

t_elem& 
VEC::operator[]( int n ) {
  if( n>=3 ) {
    cerr << "VEC error: trying to do data[>2]!" << endl;
    return data[2];		// kludge; should 'throw' an exception
  }

  return data[n];
}

t_elem dotprod( const VEC& a, const VEC& b ) {
  return a.x()*b.x() + a.y()*b.y() + a.z()*b.z();
}

// it is assumed that b is already normalized
bool 
VEC::project_on( const VEC& a, const VEC& b ) {
  double len = dotprod( a, b );
  *this = b;
  smult( len );
  return true;
}

